# BalkanID Python Library

[![pypi](https://img.shields.io/pypi/v/fern-balkan-id.svg)](https://pypi.python.org/pypi/fern-balkan-id)
[![fern shield](https://img.shields.io/badge/%F0%9F%8C%BF-SDK%20generated%20by%20Fern-brightgreen)](https://github.com/fern-api/fern)

The BalkanID Python Library provides convenient access to the Balkan-id API from applications written in Python.

## Installation

```sh
pip install --upgrade fern-balkan-id
```

## Usage

```python
from balkanid.client import BalkanID

client = BalkanID(
    api_key="YOUR_API_KEY",
    api_key_id="YOUR_API_KEY_ID",
)

response = client.campaigns.download(include_completed=True, include_draft=False)
print("Received campaigns data", response.url)
```

## Async Client

```python
import asyncio
from balkanid.client import AsyncBalkanID

client = AsyncBalkanID(
    api_key="YOUR_API_KEY",
    api_key_id="YOUR_API_KEY_ID",
)

async def get_entity() -> None:
    response = await client.campaigns.download(include_completed=True, include_draft=False)
    print("Received campaigns data", response.url)

asyncio.run(get_entity())
```

## Timeouts
By default, the client is configured to have a timeout of 60 seconds. You can customize this value at client instantiation. 

```python
from balkanid.client import BalkanID

client = BalkanID(api_key="YOUR_API_KEY", timeout=15)
```

## Handling Exceptions
All exceptions thrown by the SDK will sublcass [cohere.ApiError](./src/cohere/core/api_error.py). 

```python
from balkanid.core import ApiError

try:
  client.campaigns.download(...)
except APIError as e:  
  # handle any api related error
```

# Beta Status

This SDK is in beta, and there may be breaking changes between versions without a major 
version update. Therefore, we recommend pinning the package version to a specific version. 
This way, you can install the same version each time without breaking changes.

# Contributing

While we value open-source contributions to this SDK, this library is generated programmatically. 
Additions made directly to this library would have to be moved over to our generation code, 
otherwise they would be overwritten upon the next generated release. Feel free to open a PR as
 a proof of concept, but know that we will not be able to merge it as-is. We suggest opening 
an issue first to discuss with us!

On the other hand, contributions to the README are always very welcome!


